 /*
 /********************************************************************/
 /*                                                                  */
 /* Program name: fmct3act.dll                                       */
 /*                                                                  */
 /* Description: This module is a sample for MQSeries Workflow       */
 /*              activity implementation. It is used for the         */
 /*              sample process Order.                               */
 /*                                                                  */
 /*  Statement:     Licensed Materials - Property of IBM             */
 /*                                                                  */
 /*                 MQSeries SupportPac WA02                         */
 /*                 (C) Copyright IBM Corp. 2000                     */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*         OrderCheck  - check the Order incoming values            */
 /*         OrderError  - log the error code in the error file       */
 /*         OrderAccept - log result in the error file               */
 /*                                                                  */
 /*         all functions log their invocation with time stamp       */
 /*         in the log file.                                         */
 /*                                                                  */
 /********************************************************************/

#define MAX_LEAF_COUNT  200
#define MAX_LEVEL_DEPTH  10
#define DELIM             '$'
#define MAX_STRUCT_DEPTH   15
#define MAX_STRUCT_NAMELEN 100
#define MAX_LINE_LEN 80

#define LOG_FILE "fmct3act.log"      // log invocation, rc's, ...
#define ERROR_FILE "fmct3act.err"    // write in this file error message
                                     // when Order check return an error
                                     // and the process result (accept or
                                     // reject)

                                     // the files will be stored in the
                                     // directory where this dll is


#define MQSWF_API_ERROR      -2
#define FILE_ERROR           -1
#define SMP_ERROR             0
#define SMP_OK                1
#define SMP_CHECK_ERROR       2
#define SMP_LOOKUP_ERROR      3
#define SMP_UNKNOWN_ERROR    99

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>


#include <fmcjccon.h>                // Container API

#include <fmct3act.h>


FILE     *  logfile = 0;

/**********************************************************************/
/* Entry OrderCheck                                                   */
/* This function is implemented for the MQSeries Workflow Support Pac */
/* MAW3 sample Order process. It is invoked for the activity          */
/* Check Order. It should check the incoming container values.        */
/*                                                                    */
/**********************************************************************/

  int FMC_APIENTRY OrderCheck()
  {
    int  rc = FMC_OK;
    long lOrderRC = SMP_OK ;    // check result

    time_t           ltime          = 0;
    char             name[FMC_CONTAINER_NAME_LENGTH]      ="";
    FmcjResultHandle                 result                  = 0;
    FmcjReadOnlyContainerHandle      hdlinctnr               = 0;
    FmcjReadWriteContainerHandle     hdloutctnr              = 0;
    char                             container[4097]         ="";
    char                           * strvalbuf      = &container[0];
    int vallen = 0;
    int intrc = 0;
    int outsize = 0;
    int i;
    bool mapped = false;

    // InContainer elements, full qualified names
    char * pContainerElement_CustomerName = "PersonInfo.CustomerLastName";
    char   pCustomerName[32]="";

    char * pContainerElement_CustomerNumber = "PersonInfo.CustomerNumber";
    long   lCustomerNumber = 0;

    char * pContainerElement_PartNumber = "PartsInfo.PartNumber";
    long   lPartNumber = 0;

    char * pContainerElement_PartOrderQty = "PartsInfo.PartOrderQty";
    long   lPartOrderQty = 0;

    char * pContainerElement_PartName = "PartsInfo.PartName";
    char   pPartName[32] = "";



    // OutContainer elements, full qualified names
    char * pContainerElement_OrderRC = "Message.ReturnCode";


    // open logfile when not already opened
    if ( logfile == 0 )
    {
      logfile = fopen (LOG_FILE, "a");
      if ( logfile == 0 )
      {
         rc = FILE_ERROR ;
         return rc;
      }
    }
    // print invocation logfile
    time(&ltime);
    fprintf(logfile, "\n-----------------------------------------------------\n");
    fprintf(logfile, "%s %s", "C-API OrderCheck called: ", ctime(&ltime));
    fflush(logfile);

/*----- Init API environment -------------------------------------------*/
    rc = FmcjGlobalConnect();
    if (rc != FMC_OK)
    {
      fprintf(logfile, "API could not be initialized: %u%\n",rc);
      fflush(logfile);
      return MQSWF_API_ERROR ;
    }
/*----- Provide for detailed error checking ----------------------------*/
    result = FmcjResultObjectOfCurrentThread();
    rc = FmcjResultRc(result);
    if (rc != FMC_OK)
    {
      fprintf(logfile, "Result object could not be accessed: %u%\n",rc);
      fflush(logfile);
      return MQSWF_API_ERROR ;
    }
/*----- Get Input Container --------------------------------------------*/
    rc = FmcjContainerInContainer( &hdlinctnr );
    if (rc != FMC_OK)
    {
      fprintf(logfile, "InContainer could not be accessed: %u%\n",rc);
      fflush(logfile);
      return MQSWF_API_ERROR ;
    }
    else
    {
/*----- Get Input Container Element------------------------------------*/
      fprintf(logfile, "Input Container Name        = %s \n",
              FmcjReadOnlyContainerType(hdlinctnr, name,
                                        FMC_CONTAINER_NAME_LENGTH));

      rc = FmcjReadOnlyContainerStringValue(hdlinctnr,
                                            pContainerElement_CustomerName ,
                                            pCustomerName,
                                            sizeof(pCustomerName));
      if (rc != FMC_OK)
      {
        if( rc == FMC_ERROR_MEMBER_NOT_SET )
        {
           fprintf(logfile, "InContainer element %s value not set\n",
                pContainerElement_CustomerName );
           lOrderRC = SMP_CHECK_ERROR;
           rc = FMC_OK;
        }
        else
        {
           fprintf(logfile, "InContainer element %s accessed returns: %u\n",
                pContainerElement_CustomerName ,rc );
           rc = MQSWF_API_ERROR ;
        }
      }
      else
      {
        fprintf(logfile, "InContainer element %s : %s\n",
                pContainerElement_CustomerName , pCustomerName );
      }

      if( rc == FMC_OK)
      {
        rc = FmcjReadOnlyContainerLongValue(hdlinctnr,
                                              pContainerElement_CustomerNumber ,
                                              &lCustomerNumber);

        if (rc != FMC_OK)
        {
          if( rc == FMC_ERROR_MEMBER_NOT_SET )
          {
             fprintf(logfile, "InContainer element %s value not set\n",
                  pContainerElement_CustomerNumber );
             lOrderRC = SMP_CHECK_ERROR;
             rc = FMC_OK ;
          }
          else
          {
             fprintf(logfile, "InContainer element %s accessed returns: %u\n",
                  pContainerElement_CustomerNumber ,rc );
             rc = MQSWF_API_ERROR ;
          }
        }
        else
        {
          fprintf(logfile, "InContainer element %s : %u\n",
                  pContainerElement_CustomerNumber , lCustomerNumber );
        }
      }
      if( rc == FMC_OK)
      {
        rc = FmcjReadOnlyContainerLongValue(hdlinctnr,
                                              pContainerElement_PartNumber ,
                                              &lPartNumber);

        if (rc != FMC_OK)
        {
          if( rc == FMC_ERROR_MEMBER_NOT_SET )
          {
             fprintf(logfile, "InContainer element %s value not set\n",
                  pContainerElement_PartNumber );
             lOrderRC = SMP_CHECK_ERROR;
             rc = FMC_OK ;
          }
          else
          {
             fprintf(logfile, "InContainer element %s accessed returns: %u\n",
                  pContainerElement_PartNumber ,rc );
             rc = MQSWF_API_ERROR ;
          }
        }
        else
        {
          fprintf(logfile, "InContainer element %s : %u\n",
                  pContainerElement_PartNumber , lPartNumber );
        }
      }

      if( lOrderRC == SMP_OK)
      {
        rc = FmcjReadOnlyContainerStringValue(hdlinctnr,
                                              pContainerElement_PartName ,
                                              pPartName,
                                              sizeof(pPartName));
        if (rc != FMC_OK)
        {
          if( rc == FMC_ERROR_MEMBER_NOT_SET )
          {
             fprintf(logfile, "InContainer element %s value not set\n",
                  pContainerElement_PartName );
             lOrderRC = SMP_CHECK_ERROR;
             rc = FMC_OK ;
          }
          else
          {
             fprintf(logfile, "InContainer element %s accessed returns: %u\n",
                  pContainerElement_PartName ,rc );
             rc = MQSWF_API_ERROR ;
          }
        }
        else
        {
          fprintf(logfile, "InContainer element %s : %s\n",
                  pContainerElement_PartName , pPartName );
        }
      }

      if( rc == FMC_OK)
      {
        rc = FmcjReadOnlyContainerLongValue(hdlinctnr,
                                              pContainerElement_PartOrderQty ,
                                              &lPartOrderQty);
        if (rc != FMC_OK)
        {
          if( rc == FMC_ERROR_MEMBER_NOT_SET )
          {
             fprintf(logfile, "InContainer element %s value not set\n",
                  pContainerElement_PartOrderQty );
             lOrderRC = SMP_CHECK_ERROR;
             rc = FMC_OK;
          }
          else
          {
             fprintf(logfile, "InContainer element %s accessed returns: %u\n",
                  pContainerElement_PartOrderQty ,rc );
             rc = MQSWF_API_ERROR ;
          }
        }
        else
        {
          fprintf(logfile, "InContainer element %s : %u\n",
                  pContainerElement_PartOrderQty , lPartOrderQty );
        }
      }

      fprintf(logfile, "Container Name Value        = %s \n", name );


      rc= FmcjReadOnlyContainerDeallocate( &hdlinctnr );
      if (rc != FMC_OK)
      {
        fprintf(logfile, " ReadOnlyContainerDeallocate rc: \n",rc);
        fflush(logfile);
        rc = MQSWF_API_ERROR ;
      }
    }
  if (rc != FMC_OK)
  {
    lOrderRC = MQSWF_API_ERROR;
    fprintf(logfile, "MQSWF API error occured\n");
  }
/*----- Calculate Output Container Element------------------------------------*/
/* here is the area to do something with incontainer and set the outcontainer */
/*                                                                            */
/*----------------------------------------------------------------------------*/

  // check incoming values

  if (lOrderRC == SMP_OK)
  {
    fprintf(logfile, "check customer name string length\n");
    if ( strlen(pCustomerName) < 1)
      {
        fprintf(logfile, "no customer name specified\n");
        lOrderRC = SMP_CHECK_ERROR;
      }/* End if*/
  }


  // set the return code
  fprintf(logfile,"t:%s = %i \n",pContainerElement_OrderRC ,lOrderRC );


/*----- Set Output Container Element------------------------------------*/

  rc= FmcjContainerOutContainer( &hdloutctnr );
  if (rc != FMC_OK)
  {
    fprintf(logfile, " OutContainer rc: \n",rc);
    fflush(logfile);
    rc =  MQSWF_API_ERROR ;
  }
  else
  {


    fprintf(logfile, "Output Container Name       = %s \n",
            FmcjReadWriteContainerType(hdloutctnr, strvalbuf,
                                     FMC_CONTAINER_NAME_LENGTH));

    fprintf(logfile, "Set %s : %i \n", pContainerElement_OrderRC ,lOrderRC );
    rc= FmcjReadWriteContainerSetLongValue(hdloutctnr,
                                         pContainerElement_OrderRC ,
                                         lOrderRC );

    if (rc != FMC_OK)
    {
      fprintf(logfile, "\nWriteContainer returns rc  : %u \n", rc);
      lOrderRC = MQSWF_API_ERROR;
    }

/*----- send the output container to PEA ----------------------------------*/
    rc = FmcjContainerSetOutContainer( hdloutctnr );
    if (rc != FMC_OK)
    {
      fprintf(logfile, "\nSet Output Container - rc   =  %u\n",rc);
      fflush(logfile);
      rc = MQSWF_API_ERROR;
    }
    else
    {
      rc= FmcjReadWriteContainerDeallocate( &hdloutctnr );
      if (rc != FMC_OK)
      {
        fprintf(logfile, "\nContainerDeallocate - rc   =  %u\n",rc);
        fflush(logfile);
        rc = MQSWF_API_ERROR;
      }
    }
  }

  if (rc != FMC_OK)
    return rc;           // if MQSWF API error occured return this
  else
    return lOrderRC;     // otherwise return check result

  }

/**********************************************************************/
/* Entry OrderError                                                   */
/* This function is implemented for the MQSeries Workflow Support Pac */
/* MAW3 sample Order process. It is invoked for the activity          */
/* RejectOrder. It writes the reason of the reject in the error file. */
/*                                                                    */
/**********************************************************************/
  int FMC_APIENTRY OrderError()
  {
    int rc = FMC_OK;

    FILE          *  errorlogfile;
    time_t           ltime          = 0;
    char             name[FMC_CONTAINER_NAME_LENGTH]      ="";
    FmcjResultHandle                 result                  = 0;
    FmcjReadOnlyContainerHandle      hdlinctnr               = 0;
    FmcjReadWriteContainerHandle     hdloutctnr              = 0;
    char                             container[4097]         ="";
    char                           * strvalbuf      = &container[0];
    int vallen = 0;
    int intrc = 0;
    char pMsgText[30] = "" ;

    // InContainer Elements

    char * pContainerElement_ReturnCode = "Message.ReturnCode";
    long  lOrderRC = SMP_OK;

    // open log file if not already opened
    if ( logfile == 0 )
    {
      logfile = fopen (LOG_FILE, "a");
      if ( logfile == 0 )
      {
         rc = FILE_ERROR ;
         return rc;
      }
    }
    time(&ltime);
    fprintf(logfile, "\n-----------------------------------------------------\n");
    fprintf(logfile, "%s %s", "C-API Activity OrderError called: ", ctime(&ltime));
    fflush(logfile);

/*----- Init API environment -------------------------------------------*/
    rc = FmcjGlobalConnect();
    if (rc != FMC_OK)
    {
      fprintf(logfile, "API could not be initialized: %u%\n",rc);
      return MQSWF_API_ERROR;
    }
/*----- Provide for detailed error checking ----------------------------*/
    result = FmcjResultObjectOfCurrentThread();
    rc = FmcjResultRc(result);
    if (rc != FMC_OK)
    {
      fprintf(logfile, "Result object could not be accessed: %u%\n",rc);
      return MQSWF_API_ERROR;
    }
/*----- Get Input Container --------------------------------------------*/
    rc = FmcjContainerInContainer( &hdlinctnr );
    if (rc != FMC_OK)
    {
      fprintf(logfile, "\nGet Input Container - rc       =  %u \n", rc);
      fflush(logfile);
      return MQSWF_API_ERROR;
    }
    else
    {
/*----- Get Input Container Element------------------------------------*/

      fprintf(logfile, "Input Container Name           =  %s \n",
              FmcjReadOnlyContainerType(hdlinctnr, name,
                                        FMC_CONTAINER_NAME_LENGTH));

      rc = FmcjReadOnlyContainerLongValue(hdlinctnr,
                                            pContainerElement_ReturnCode ,
                                            &lOrderRC);
      if (rc != FMC_OK)
      {
        if( rc == FMC_ERROR_MEMBER_NOT_SET )
        {
           fprintf(logfile, "InContainer element %s value not set\n",
                pContainerElement_ReturnCode );
           lOrderRC = SMP_ERROR;
           rc = FMC_OK;
        }
        else
        {
           fprintf(logfile, "InContainer element %s accessed returns: %u\n",
                pContainerElement_ReturnCode ,rc );
           rc = MQSWF_API_ERROR ;
        }
      }
      else
      {
        fprintf(logfile, "InContainer element %s : %u\n",
                pContainerElement_ReturnCode , lOrderRC );
      }

      rc= FmcjReadOnlyContainerDeallocate( &hdlinctnr );
      if (rc != FMC_OK)
      {
        fprintf(logfile, "\nReadOnlyContainerDeallocate - rc =  %u \n", rc);
        fflush(logfile);
        rc =  MQSWF_API_ERROR;
      }
    }

/*----- Calculate Output Container Element------------------------------------*/


    switch ( lOrderRC )
      {
        case MQSWF_API_ERROR :
          strcpy(pMsgText,"MQSWF API error\n ");
          break;
        case FILE_ERROR :
          strcpy(pMsgText,"log file error\n ");
          break;
        case SMP_ERROR :
          strcpy(pMsgText,"Required value not set\n ");
          break;
        case SMP_CHECK_ERROR :
          strcpy(pMsgText,"Check error, required value not set\n ");
          break;
        case SMP_LOOKUP_ERROR :
          strcpy(pMsgText,"Parts not available\n ");
          break;
        default:
          strcpy(pMsgText,"unknown check error\n");
          break;
      }/* End switch*/

    errorlogfile = fopen (ERROR_FILE, "a");
    if ( errorlogfile == 0 )
    {
       rc = FILE_ERROR ;
    }
    else
    {
      time(&ltime);
      fprintf(errorlogfile, "\n-----------------------------------------------------\n");
      fprintf(errorlogfile, "%s %s", "CheckOrder returned an error \n ", ctime(&ltime) );
      fprintf(errorlogfile, "CheckORder rc: %d, error : %s \n ", lOrderRC, pMsgText );
      fflush(errorlogfile);
      fclose(errorlogfile);
    }
    return rc;
  }

/**********************************************************************/
/* Entry OrderAccept                                                  */
/* This function is implemented for the MQSeries Workflow Support Pac */
/* MAW3 sample Order process. It is invoked for the activity          */
/* Accept Order. It writes the result also in the error file.         */
/*                                                                    */
/**********************************************************************/
  int FMC_APIENTRY OrderAccept()
  {
    int rc = FMC_OK;

    FILE          *  errorlogfile;
    time_t           ltime          = 0;
    char             name[FMC_CONTAINER_NAME_LENGTH]      ="";
    FmcjResultHandle                 result                  = 0;
    FmcjReadOnlyContainerHandle      hdlinctnr               = 0;
    FmcjReadWriteContainerHandle     hdloutctnr              = 0;
    char                             container[4097]         ="";
    char                           * strvalbuf      = &container[0];
    int vallen = 0;
    int intrc = 0;
    char pMsgText[30] = "" ;

    // InContainer Elements

    char * pContainerElement_ReturnCode = "Message.ReturnCode";
    long  lOrderRC = SMP_OK;

    // open log file if not already opened
    if ( logfile == 0 )
    {
      logfile = fopen (LOG_FILE, "a");
      if ( logfile == 0 )
      {
         rc = FILE_ERROR ;
         return rc;
      }
    }

    time(&ltime);
    fprintf(logfile, "\n-----------------------------------------------------\n");
    fprintf(logfile, "%s %s", "C-API Activity OrderAccept called: ", ctime(&ltime));
    fflush(logfile);

/*----- Init API environment -------------------------------------------*/
    rc = FmcjGlobalConnect();
    if (rc != FMC_OK)
    {
      fprintf(logfile, "API could not be initialized: %u%\n",rc);
      return MQSWF_API_ERROR;
    }
/*----- Provide for detailed error checking ----------------------------*/
    result = FmcjResultObjectOfCurrentThread();
    rc = FmcjResultRc(result);
    if (rc != FMC_OK)
    {
      fprintf(logfile, "Result object could not be accessed: %u%\n",rc);
    }
/*----- Get Input Container --------------------------------------------*/
    rc = FmcjContainerInContainer( &hdlinctnr );
    if (rc != FMC_OK)
    {
      fprintf(logfile, "\nGet Input Container - rc       =  %u \n", rc);
      fflush(logfile);
      return MQSWF_API_ERROR;
    }
    else
    {

/*----- Get Input Container Element------------------------------------*/
      fprintf(logfile, "Input Container Name           =  %s \n",
              FmcjReadOnlyContainerType(hdlinctnr, name,
                                        FMC_CONTAINER_NAME_LENGTH));

      rc = FmcjReadOnlyContainerLongValue(hdlinctnr,
                                            pContainerElement_ReturnCode ,
                                            &lOrderRC);
      if (rc != FMC_OK)
      {
        if( rc == FMC_ERROR_MEMBER_NOT_SET )
        {
           fprintf(logfile, "InContainer element %s value not set\n",
                pContainerElement_ReturnCode );
           lOrderRC = SMP_ERROR;
           rc = FMC_OK;
        }
        else
        {
           fprintf(logfile, "InContainer element %s accessed returns: %u\n",
                pContainerElement_ReturnCode ,rc );
           rc = MQSWF_API_ERROR ;
        }
      }
      else
      {
        fprintf(logfile, "InContainer element %s : %u\n",
                pContainerElement_ReturnCode , lOrderRC );
      }

      rc= FmcjReadOnlyContainerDeallocate( &hdlinctnr );
      if (rc != FMC_OK)
      {
        fprintf(logfile, "\nReadOnlyContainerDeallocate - rc =  %u \n", rc);
        fflush(logfile);
      }
    }
/*----- Calculate Output Container Element------------------------------------*/
    switch ( lOrderRC )
      {
        case SMP_OK :
          strcpy(pMsgText,"Order accept\n ");
          break;
        default:
          strcpy(pMsgText,"unknown error\n");
          break;
      }/* End switch*/

    errorlogfile = fopen (ERROR_FILE, "a");
    if ( errorlogfile == 0 )
    {
       rc = FILE_ERROR ;
    }
    else
    {
      time(&ltime);
      fprintf(errorlogfile, "\n-----------------------------------------------------\n");
      fprintf(errorlogfile, "%s %s", "OrderAccept \n ", ctime(&ltime) );
      fprintf(errorlogfile, "OrderAccept rc: %d, error : %s \n ", lOrderRC, pMsgText );
      fflush(errorlogfile);
      fclose(errorlogfile);
    }
    return rc;
  }


/**********************************************************************/
/**********************************************************************/
