 /*
 /********************************************************************/
 /*                                                                  */
 /* Program name: lookup                                             */
 /*                                                                  */
 /* Description: Sample C program used for the Order Process.        */
 /*              It waits until a message arrives in a queue with    */
 /*              Order PartNumber, PartOrderQuantity and PartName    */
 /*              It checks for availablity and returns a returncode  */
 /*              ReturnCode 1 means available others means not       */
 /*              available or error.                                 */
 /*                                                                  */
 /*  Statement:     Licensed Materials - Property of IBM             */
 /*                                                                  */
 /*                 MQSeries SupportPac WA02                         */
 /*                 (C) Copyright IBM Corp. 2000                     */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*                                                                  */
 /*    Program logic:                                                */
 /*         MQOPEN target queue for OUTPUT                           */
 /*         .  MQGET  C-Structure                                    */
 /*         .  MQPUT1 C-Structure                                    */
 /*         MQCLOSE target queue                                     */
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /*   lookup has 2 parameters                                        */
 /*                 - the name of the target queue (required)        */
 /*                 - queue manager name (optional)                  */
 /*                                                                  */
 /********************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
    /* includes for MQI */
 #include <cmqc.h>
    /* order message structure */
 #include <fmct3act.h>

 int main(int argc, char **argv)
 {
   /*   Declare MQI structures needed                                */
   MQOD    od = {MQOD_DEFAULT};     /* Object Descriptor             */
   MQOD    odR = {MQOD_DEFAULT};    /* Object Descriptor reply       */
   MQMD    md = {MQMD_DEFAULT};     /* Message Descriptor            */
   MQMD    mdR = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQGMO   gmo = {MQGMO_DEFAULT};   /* get message options           */
   MQPMO   pmo = {MQPMO_DEFAULT};   /* put message options           */

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONN        */

   MQLONG   messlen;                /* message length                */
   char     buffer[2000];           /* message buffer                */
   char     Repbuffer[2000];        /* message reply buffer          */
   char     QMName[50];             /* queue manager name            */
   int      i,j,k;

   char sTmpPartName[32]="";

   Order_PartsInfo *pPartsInfo;     /* recieving C Structure         */
   Order_Message   sMessage;        /* replying C Structure          */

   printf("### sample lookup start\n");
   if (argc < 2)
   {
     printf("Error: Required parameter missing - queue name\n");
     printf("\nInvoke Lookup <queue name> <queue manager name>\n");
     exit(99);
   }

   /******************************************************************/
   /*                                                                */
   /*   Connect to queue manager                                     */
   /*                                                                */
   /******************************************************************/
   QMName[0] = 0;    /* default */
   if (argc > 2)
     strcpy(QMName, argv[2]);
   MQCONN(QMName,                   /* queue manager                 */
          &Hcon,                    /* connection handle             */
          &CompCode,                /* completion code               */
          &CReason);                /* reason code                   */

   /* report reason and stop if it failed     */
   if (CompCode == MQCC_FAILED)
   {
     printf("Error: MQCONN ended with reason code %ld\n", CReason);
     exit( (int)CReason );
   }

   /******************************************************************/
   /*                                                                */
   /*   Use parameter as the name of the source queue                */
   /*                                                                */
   /******************************************************************/
   strncpy(od.ObjectName, argv[1], (size_t)MQ_Q_NAME_LENGTH);

   /******************************************************************/
   /*                                                                */
   /*   Open the target message queue for output                     */
   /*                                                                */
   /******************************************************************/
   O_options = MQOO_INPUT_SHARED    /* open queue for shared         */
           + MQOO_FAIL_IF_QUIESCING;/* but not if MQM stopping       */
   MQOPEN(Hcon,                     /* connection handle             */
          &od,                      /* object descriptor for queue   */
          O_options,                /* open options                  */
          &Hobj,                    /* object handle                 */
          &OpenCode,                /* MQOPEN completion code        */
          &Reason);                 /* reason code                   */

   /* report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
   {
     printf("Error: MQOPEN ended with reason code %ld\n", Reason);
     return( Reason);
   }

   if (OpenCode == MQCC_FAILED)
   {
     printf("Error: unable to open queue for output\n");
     return( Reason);
   }

   /******************************************************************/
   /*                                                                */
   /* now look until program terminates or until error occure        */
   /*                                                                */
   /******************************************************************/
   k=0;
   while (CompCode != MQCC_FAILED)
   {
     k++;
     printf("Now wait (%d) for message in %s ... \n\n",k,od.ObjectName);

     memset(md.MsgId, '\0',24);
     memset(md.CorrelId, '\0',24);

     gmo.Options= MQGMO_WAIT;
     gmo.Options |= MQGMO_NO_SYNCPOINT;
     gmo.Options |= MQGMO_FAIL_IF_QUIESCING;
     gmo.WaitInterval= MQWI_UNLIMITED;

     messlen = sizeof(buffer);
     memset(buffer, ' ',sizeof(buffer));


     /****************************************************************/
     /*                                                              */
     /*                                                              */
     /****************************************************************/
     MQGET(Hcon,                    /* connection handle             */
           Hobj,                    /* object handle                 */
           &md,                     /* message descriptor            */
           &gmo,                    /* GET options                   */
           messlen,                 /* buffer length                 */
           &buffer,                 /* message buffer                */
           &messlen,                /* message length                */
           &CompCode,               /* completion code               */
           &Reason);                /* reason code                   */

     printf("### MQGET, message arrived. Reason code=%ld, messlen=%d \n",
            Reason, messlen);


     /* report reason if any                                         */
     if (Reason != MQRC_NONE)
      {
       printf("Error: MQGET ended with reason code %ld\n", Reason);
       return(-1);                   /* MQGET failed                 */
      };


     /****************************************************************/
     /*                                                              */
     /*  now handle arrived message                                  */
     /*                                                              */
     /*  set output value                                            */
     /*                                                              */
     /****************************************************************/
     pPartsInfo = (void *)buffer;

     printf("  Received Message PartsInfo:\n"\
            "   PartsInfo.PartNumber: %d,\n"\
            "   PartsInfo.PartOrderQty: %d,\n"\
            "   PartsInfo.PartName: %s \n",
            pPartsInfo->PartNumber,
            pPartsInfo->PartOrderQty,
            pPartsInfo->PartName);


     /* and further processing of the input message can be done here */


     /* in this sample, assign fix the return code                   */

     sMessage.ReturnCode = 1;


     /****************************************************************/
     /*                                                              */
     /*  build and put reply message                                 */
     /*                                                              */
     /****************************************************************/

     CompCode = OpenCode;           /* use MQOPEN result             */

     mdR.MsgType = MQMT_REPLY;
     mdR.Persistence = MQPER_PERSISTENT;
     /* the correlation id is used to tie the msg. to the reply      */
     memcpy(mdR.CorrelId, md.MsgId, sizeof(mdR.CorrelId));
     memcpy(mdR.Format,             /* character string format       */
            MQFMT_STRING, (size_t)MQ_FORMAT_LENGTH);
     memcpy(mdR.MsgId,              /* reset MsgId to get a new one  */
            MQMI_NONE, sizeof(mdR.MsgId) );

     memcpy(odR.ObjectName, md.ReplyToQ, MQ_Q_NAME_LENGTH);
     memcpy(odR.ObjectQMgrName, md.ReplyToQMgr, MQ_Q_MGR_NAME_LENGTH);

     strcpy(md.ReplyToQ,"WfSmpError");

     j = sizeof(Repbuffer);
     memset(Repbuffer, ' ', j);

     memcpy(buffer, (void *)&sMessage, sizeof(sMessage));

     messlen = sizeof(sMessage);


     printf("### MQPUT to reply queue=%48.48s\n"\
            "             reply qmgr=%48.48s \n"\
            "             messlen=%d\n",
             odR.ObjectName,odR.ObjectQMgrName,messlen);

     printf("    Put C Struct:  Message.ReturnCode: %d,\n",
                                            sMessage.ReturnCode);

     /****************************************************************/
     /* put the result on the reply to queue                         */
     /*                                                              */
     /*                                                              */
     /****************************************************************/

     MQPUT1(Hcon,                   /* connection handle             */
           &odR,                    /* object descriptor             */
           &mdR,                    /* message descriptor            */
           &pmo,                    /* default options (datagram)    */
           messlen,                 /* message length                */
           buffer,                  /* message buffer                */
           &CompCode,               /* completion code               */
           &Reason);                /* reason code                   */

     /* report reason, if any */
     if (Reason != MQRC_NONE)
     {
       printf("Error: MQPUT ended with reason code %ld\n", Reason);
       CompCode = MQCC_FAILED;
     }
   }   /* end while */
   /******************************************************************/
   /*                                                                */
   /*   Close the target queue (if it was opened)                    */
   /*                                                                */
   /******************************************************************/
   if (CompCode != MQCC_FAILED)
   {
     C_options = 0;                 /* no close options              */
     MQCLOSE(Hcon,                  /* connection handle             */
             &Hobj,                 /* object handle                 */
             C_options,
             &CompCode,             /* completion code               */
             &Reason);              /* reason code                   */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("Error: MQCLOSE ended with reason code %ld\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /*   Disconnect from MQM if not already connected                 */
   /*                                                                */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
     MQDISC(&Hcon,                  /* connection handle             */
            &CompCode,              /* completion code               */
            &Reason);               /* reason code                   */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("Error: MQDISC ended with reason code %ld\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /* END OF AMQSPUT0                                                */
   /*                                                                */
   /******************************************************************/
   printf("### sample Lookup end\n");
   return(0);
 }
